Package: thematic
Title: Unified and Automatic 'Theming' of 'ggplot2', 'lattice', and
        'base' R Graphics
Version: 0.1.8
Authors@R: c(
    person("Carson", "Sievert", , "carson@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4958-2844")),
    person("Barret", "Schloerke", , "barret@posit.co", role = "aut",
           comment = c(ORCID = "0000-0001-9986-114X")),
    person("Joe", "Cheng", , "joe@posit.co", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Theme 'ggplot2', 'lattice', and 'base' graphics based on a
    few choices, including foreground color, background color, accent
    color, and font family. Fonts that aren't available on the system, but
    are available via download on 'Google Fonts', can be automatically
    downloaded, cached, and registered for use with the 'showtext' and
    'ragg' packages.
License: MIT + file LICENSE
URL: https://rstudio.github.io/thematic/,
        https://github.com/rstudio/thematic
BugReports: https://github.com/rstudio/thematic/issues
Depends: R (>= 3.0.0)
Imports: farver, ggplot2 (>= 3.5.2), graphics, grDevices, grid,
        rappdirs, rlang, rstudioapi (>= 0.8), scales, utils
Suggests: bslib, Cairo, curl, gganimate, ggiraph, htmltools, jsonlite,
        knitr, lattice, ragg, rmarkdown, shiny (>= 1.5.0), showtext,
        stats, svglite, sysfonts, systemfonts, testthat, vdiffr (>=
        1.0.0), withr
Config/Needs/check: shinytest2, callr, sf, ggthemes, patchwork,
        gridExtra, tinytex, devtools, rversions
Config/Needs/routine: ggrepel, r-lib/evaluate
Config/Needs/website: GGally, RColorBrewer, patchwork,
        apreshill/quillt, r-lib/evaluate
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.3
Collate: 'auto.R' 'base.R' 'cache.R' 'gfonts.R' 'ggplot.R' 'globals.R'
        'hooks.R' 'knitr.R' 'lattice.R' 'onLoad.R' 'thematic-package.R'
        'thematic-save-plot.R' 'thematic.R' 'utils.R'
NeedsCompilation: no
Packaged: 2025-09-29 22:13:21 UTC; cpsievert
Author: Carson Sievert [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4958-2844>),
  Barret Schloerke [aut] (ORCID: <https://orcid.org/0000-0001-9986-114X>),
  Joe Cheng [aut],
  Posit Software, PBC [cph, fnd]
Maintainer: Carson Sievert <carson@posit.co>
Repository: CRAN
Date/Publication: 2025-09-29 23:40:03 UTC
Built: R 4.6.0; ; 2026-04-28 07:34:56 UTC; unix
