# Generated by roxygen2: do not edit by hand

export(auto_config)
export(auto_config_get)
export(auto_config_set)
export(auto_resolve_theme)
export(default_device)
export(font_cache_set)
export(font_spec)
export(okabe_ito)
export(sequential_gradient)
export(thematic_get_mixture)
export(thematic_get_option)
export(thematic_get_theme)
export(thematic_local_theme)
export(thematic_off)
export(thematic_on)
export(thematic_rmd)
export(thematic_save_plot)
export(thematic_set_theme)
export(thematic_shiny)
export(thematic_theme)
export(thematic_with_theme)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.new)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev.set)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,palette)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(grid,grid.text)
importFrom(grid,pushViewport)
importFrom(grid,seekViewport)
importFrom(grid,upViewport)
importFrom(rlang,"%|%")
importFrom(rlang,is_installed)
importFrom(rlang,list2)
importFrom(rstudioapi,getThemeInfo)
importFrom(rstudioapi,readRStudioPreference)
importFrom(rstudioapi,versionInfo)
importFrom(utils,assignInNamespace)
importFrom(utils,capture.output)
importFrom(utils,download.file)
importFrom(utils,getFromNamespace)
importFrom(utils,getS3method)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,unzip)
