#' @importFrom graphics par plot plot.new
#' @importFrom grid grid.newpage grid.draw grid.text gpar seekViewport pushViewport upViewport
#' @importFrom grDevices col2rgb dev.new dev.cur dev.off dev.set devAskNewPage palette colorRampPalette
#' @importFrom utils capture.output download.file unzip packageVersion modifyList getFromNamespace getS3method assignInNamespace
#' @importFrom rlang is_installed list2 %|%
#' @importFrom rstudioapi getThemeInfo versionInfo readRStudioPreference
NULL
