---
title: "Untitled"
output: 
  html_document_base:
    theme: 
      bootswatch: {{theme}}
    self_contained: false
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  echo = FALSE,
  fig.path = "./"
)
thematic::thematic_rmd()
```

```{r ggplot}
library(ggplot2)
qplot(x = 1:10, y = 1:10, color = 1:10)
```

```{r lattice}
lattice::show.settings()
```

```{r base}
image(volcano, col = thematic_get_option("sequential"))
```
