---
output: pdf_document
---

```{r}
library(thematic)
thematic_rmd(
  "#444444", "#e4e4e4", "#749886", 
  font = font_spec("Liu Jian Mao Cao", scale = 1.25, update = TRUE)
)
```

```{r ggplot}
library(ggplot2)
qplot(x = 1:10, y = 1:10, color = 1:10)
```

```{r lattice}
lattice::show.settings()
```

```{r base}
image(volcano, col = thematic_get_option("sequential", resolve = FALSE))
```
